# SPDX-FileCopyrightText : 2023 Detlef Gebhardt, written for CircuitPython
# SPDX-FileCopyrightText : Copyright (c) 2023 Detlef Gebhardt
# SPDX-Filename          : Earth-Clock  (10.9.2023)
# SPDX-License-Identifier: GEBMEDIA
import time
import rtc
import gc
import board
import math
import busio
import displayio
import digitalio
import terminalio
import bitmaptools
from adafruit_display_shapes.roundrect import RoundRect
from adafruit_display_shapes.circle import Circle
from adafruit_display_text import label
import gc9a01
import my_qmi8658
import adafruit_imageload
from adafruit_ticks import ticks_ms

# Release any resources currently in use for the displays
displayio.release_displays()

# Make the displayio SPI bus and the GC9A01 display
spi = busio.SPI(clock=board.LCD_CLK, MOSI=board.LCD_DIN)
display_bus = displayio.FourWire(spi, command=board.LCD_DC, chip_select=board.LCD_CS, reset=board.LCD_RST)
display = gc9a01.GC9A01(display_bus, width=240, height=240, backlight_pin=board.LCD_BL)
display.rotation = 90
display.brightness = 1

group1 = displayio.Group()
main = displayio.Group()

# Sensor initialisieren
sensor=my_qmi8658.QMI8658()

# Variable definieren
text = "pico - watch"
#text = "pico-watch\n    by\n gebmedia"
zeit = "00:00:00"
start = 0

# Bitmap-Dateien für Hintergrund und Zeiger
zifferblatt = "/images/zifferblatt.bmp"
second_zeiger = "/images/second.bmp"
minute_zeiger = "/images/min_zeiger.bmp"
hour_zeiger = "/images/hour_zeiger.bmp"

# Rechteck mit abgerundeten Ecken
roundrect1 = RoundRect(60, 132, 120, 40, 10, fill=0x0, outline=0xffffff, stroke=3)
group1.append(roundrect1)

## Group -group1- zum Stellen der Uhr
# create the label
updating_label1 = label.Label(
    font=terminalio.FONT, text=zeit, scale=2, color=0xffffff,
    line_spacing=1)
# set label position and add label
updating_label1.anchor_point = (0, 0)
updating_label1.anchored_position = (70, 140)
group1.append(updating_label1)

## Group -main- Anzeige der analogen Zeit
# Ziffernblatt als Hintergrund
bg_bitmap,bg_pal = adafruit_imageload.load(zifferblatt)
bg_tile_grid = displayio.TileGrid(bg_bitmap, pixel_shader=bg_pal)
main.append(bg_tile_grid)
# Text
text_area = label.Label(terminalio.FONT, text=text, line_spacing=0.9, color=0xffffff, anchor_point=(0.5,0.5), anchored_position=(0,0))
text_group = displayio.Group(scale=1, x=120, y=80)
text_group.append(text_area)
main.append(text_group)
# Stundenzeiger 30x140
bitmap_pointer_hour, palette_pointer = adafruit_imageload.load(hour_zeiger, bitmap=displayio.Bitmap,palette=displayio.Palette)
palette_pointer.make_transparent(0)
# Blank bitmap vom Stundenzeiger
bitmap_pointer_blank_hour = displayio.Bitmap(bitmap_pointer_hour.width, bitmap_pointer_hour.height, 1)
# Minutenzeiger 30x140
bitmap_pointer_min, palette_pointer = adafruit_imageload.load(minute_zeiger, bitmap=displayio.Bitmap,palette=displayio.Palette)
palette_pointer.make_transparent(0)
# Blank bitmap vom Minutenzeiger
bitmap_pointer_blank_min = displayio.Bitmap(bitmap_pointer_min.width, bitmap_pointer_min.height, 1)
# Sekundenzeiger 30x140 pointer
bitmap_pointer_sec, palette_pointer = adafruit_imageload.load(second_zeiger, bitmap=displayio.Bitmap,palette=displayio.Palette)
palette_pointer.make_transparent(0)
# Blank bitmap vom Sekundenzeiger
bitmap_pointer_blank_sec = displayio.Bitmap(bitmap_pointer_sec.width, bitmap_pointer_sec.height, 1)
# Transparentes Overlay für 'rotozoom'
# Zeiger für die Drehung
bitmap_scribble_hour = displayio.Bitmap(display.width, display.height, len(palette_pointer))
tile_grid = displayio.TileGrid(bitmap_scribble_hour, pixel_shader=palette_pointer)
main.append(tile_grid)
bitmap_scribble_min = displayio.Bitmap(display.width, display.height, len(palette_pointer))
tile_grid = displayio.TileGrid(bitmap_scribble_min, pixel_shader=palette_pointer)
main.append(tile_grid)
bitmap_scribble_sec = displayio.Bitmap(display.width, display.height, len(palette_pointer))
tile_grid = displayio.TileGrid(bitmap_scribble_sec, pixel_shader=palette_pointer)
main.append(tile_grid)
circle1 = Circle(120, 120, 10, fill=0xff0000, outline=None)
main.append(circle1)
circle2 = Circle(120, 120, 5, fill=0x000000, outline=0x0)
main.append(circle2)

current_time = time.localtime()
hour = current_time.tm_hour
minute = current_time.tm_min
second = current_time.tm_sec

if current_time.tm_min == 0 and current_time.tm_hour == 0:
    stellen = True
    display.show(group1)
else:
    stellen = False
    ein = True
    display.show(main)
    start = ticks_ms()

# zum Testen Zeit einstellen
#stellen = True
#display.show(group1)

while True:
    #read QMI8658
    reading=sensor.Read_XYZ()   
    # Display wird um die x- und y-Achse gekippt
    wert_y = (10)*reading[0]
    wert_x = (10)*reading[1]
    if stellen == True:
        # Uhrzeit einstellen
        if wert_y < -6:
            display.rotation = 0
            start = ticks_ms()
            if minute < 59:
                minute += 1
                time.sleep(0.6)
            else:
                minute = 0
                time.sleep(0.6)
            r = rtc.RTC()
            r.datetime = time.struct_time((2023, 1, 1, hour, minute, 0, 6, 1, -1))
        if wert_y > 6:
            display.rotation = 180
            start = ticks_ms()
            if hour < 23:
                hour += 1
                time.sleep(0.6)
            else:
                hour = 0
                time.sleep(0.6)
            r = rtc.RTC()
            r.datetime = time.struct_time((2023, 1, 1, hour, minute, 0, 6, 1, -1))
        # Uhrzeit einstellen abschliessen
        if wert_y > -2 and wert_y < 2 and wert_x > -8 and wert_x < 8:
            display.rotation = 90
            if (ticks_ms() - start)/ 1000 > 7:
                stellen = False
                updating_label1.text = "fertig"
                time.sleep(2)
                display.refresh()
                display.show(main)
        #eingestellte Zeit holen und anzeigen
        current_time = time.localtime()
        hour = current_time.tm_hour
        minute = current_time.tm_min
        second = current_time.tm_sec
        zeit = str(hour) + ":" + str(minute) + ":" + str(second)
        if second < 10:
            zeit = str(hour) + ":" + str(minute) + ":" + str(second)
        if minute < 10:
            zeit = str(hour) + ":" + "0" + str(minute) + ":" + str(second)
        if second < 10 and minute < 10:
            zeit = str(hour) + ":" + "0" + str(minute) + ":" + str(second)
        updating_label1.text = zeit
    else:
        #print(wert_y,",",wert_x)
        # reagiert bei kurzer Bewegung am linken Arm
        if wert_x < -4:
            display.brightness = 1
            start = ticks_ms()
        # nach 7s wieder ausschalten
        if (ticks_ms() - start)/ 1000 > 7:
            display.brightness = 0.01
                
        # Analogzeit anzeigen
        current_time = time.localtime()
        second = current_time.tm_sec
        alpha_rad_sec = math.pi/30 * second
        bitmaptools.rotozoom( bitmap_scribble_sec, bitmap_pointer_sec, angle = alpha_rad_sec, px=15,py=107)
        alpha_rad_hour = math.pi/6 * hour + math.pi/180 * minute/2
        bitmaptools.rotozoom( bitmap_scribble_hour, bitmap_pointer_hour, angle = alpha_rad_hour, px=15,py=120)
        alpha_rad_min = math.pi/30 * minute
        bitmaptools.rotozoom( bitmap_scribble_min, bitmap_pointer_min, angle = alpha_rad_min, px=15,py=120)
        if second == 59:
            bitmaptools.rotozoom( bitmap_scribble_sec, bitmap_pointer_blank_sec, angle = alpha_rad_sec, px=15,py=107)
            minute += 1
            if minute == 60:
                minute = 0
                hour += 1
                if hour == 24:
                    hour = 0
            display.refresh()
            time.sleep(0.6)
    
    gc.collect()   
    #print(gc.mem_free())